import importlib
import os
import time
from uuid import uuid4

from flask import Flask, request, g

from api.route import Route
from core.logger import APILogger

app = Flask(__name__)
logger = APILogger(__name__)


def filter_relevant_headers(headers: dict) -> dict:
    relevant_keys = {
        "Content-Type",
        "Host",
        "Connection",
        "User-Agent",
        "Origin",
        "Referer",
        "Accept",
    }
    return {key: value for key, value in headers.items() if key in relevant_keys}


@app.before_request
async def log_request():
    g.request_id = uuid4()
    g.start_time = time.time()
    logger.debug(
        f"Request {g.request_id}: {request.method}@{request.path} from {request.remote_addr}"
    )
    user = await Route.get_user()

    request_info = {
        "headers": filter_relevant_headers(dict(request.headers)),
        "args": request.args.to_dict(),
        "form-data": request.form.to_dict(),
        "payload": request.get_json(silent=True),
        "user": f"{user.id}-{user.keycloak_id}" if user else None,
        "files": (
            {key: file.filename for key, file in request.files.items()}
            if request.files
            else None
        ),
    }

    logger.trace(f"Request {g.request_id}: {request_info}")


@app.after_request
def log_after_request(response):
    # calc the time it took to process the request
    duration = (time.time() - g.start_time) * 1000
    logger.info(
        f"Request finished {g.request_id}: {response.status_code}-{request.method}@{request.path} from {request.remote_addr} in {duration:.2f}ms"
    )
    return response


@app.errorhandler(Exception)
def handle_exception(e):
    logger.error(f"Request {g.request_id}", e)
    return {"error": str(e)}, 500


# used to import all routes
routes_dir = os.path.join(os.path.dirname(__file__), "routes")
for filename in os.listdir(routes_dir):
    if filename.endswith(".py") and filename != "__init__.py":
        module_name = f"api.routes.{filename[:-3]}"
        importlib.import_module(module_name)

# Explicitly register the routes
for route, (view_func, options) in Route.registered_routes.items():
    app.add_url_rule(route, view_func=view_func, **options)
